@@ -284,6 +284,8 @@ class Client:
 
         self.exch_active = agent_running
 
+        self.is_sending = self.exch_active
+
         self.isRegistered = False
 
         self.polling_interval = 3
@@ -292,7 +294,7 @@ class Client:
 
         self.max_iterations = max_round if max_round > 0 else 10000
 
-        self.num_samples = 1
+        self.agg_weight = 1
 
         self.register_agent()
 
@@ -379,8 +381,8 @@ class Client:
         try:
             data_dict, performance_dict = load_model_file(
                 self.model_path, self.lmfile)
-        except:
-            self.logger.error("Cannot load local models!")
+        except Exception as e:
+            self.logger.error(f"Cannot load local models!\n{e}")
             return
 
         rand_id, gene_time, models, model_id, self.version, base_model = \
@@ -391,6 +393,7 @@ class Client:
             # Create an initialize model message
             msg = generate_initialize_model_message(self, model_id, models, gene_time, performance_dict,
                                                     base_model)
+
             resp = await self.comm_handler.send_msg(msg, self.reg_addr, resp_required=True)
 
             if resp[0] == AggMsgType.max_size_reached:
@@ -432,7 +435,6 @@ class Client:
             self.msend_addr = f'{self.aggr_ip}:{self.msend_port}'
 
         while self.exch_active:
-
             # Number of iterations if clearly specified by users
             if self.is_finite == True and loop_count == self.max_iterations:
                 self.agent_running = False
@@ -458,7 +460,7 @@ class Client:
                     data_dict)
 
                 upd_msg = generate_lmodel_update_message(
-                    self.id, model_id, models, performance_dict, self.num_samples)
+                    self.id, model_id, models, performance_dict, self.agg_weight)
 
                 try:
                     resp = await self.comm_handler.send_msg(upd_msg, self.msend_addr, resp_required=True)
@@ -499,14 +501,17 @@ class Client:
                 msg.append(AgentMsgType.waiting_sgm)  # 0
                 msg.append(self.round)
                 msg.append(self.id)
-                msg.append(self.exch_active)
+                msg.append((self.exch_active and self.is_sending))
 
                 try:
                     resp = await self.comm_handler.send_msg(msg, self.msend_addr)
 
                     if resp[0] == AggMsgType.update:
-                        self.logger.debug(f'--- SG Model Received ---')
-                        self.save_model_from_message(resp)
+                        if (self.round < resp[int(SGMDistributionMsgLocation.round)]):
+                            self.logger.debug(f'--- SG Model Received ---')
+                            self.save_model_from_message(resp)
+                        else:
+                            self.logger.debug('Old poll response received, discarding')
 
                     elif resp[0] == AggMsgType.max_agents_reached:
                         self.logger.error(f"--- Aggregator Response: {resp[1]}---")
@@ -540,6 +545,7 @@ class Client:
             SGMDistributionMsgLocation.model_id)]
         data_dict['models'] = sgm_msg[int(
             SGMDistributionMsgLocation.sg_models)]
+
         self.round = sgm_msg[int(SGMDistributionMsgLocation.round)]
 
         # Save the received SG models to the local file
@@ -677,6 +683,13 @@ class Client:
         # send model by writing sending state
         write_state(model_path, self.statefile, ClientState.sending)
 
+    def send_metrics(self, model_id, metrics, metric_round=None):
+        msg = [AgentMsgType.send_metrics, model_id, metrics, metric_round]
+
+        loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(loop)
+        asyncio.get_event_loop().run_until_complete(self.comm_handler.send_msg(msg, self.msend_addr, resp_required=False))
+
     def read_agent_state(self):
         state = read_state(self.model_path, self.statefile)
         return state
@@ -718,12 +731,13 @@ class Client:
             'models': base_model.weights,
             'base_model': base_model
         }
+
         self.save_model(model_dict)
 
         self.logger.info(f"Base model prepared for upload")
 
-    def set_num_samples(self, num_samples):
-        self.num_samples = num_samples
+    def set_agg_weight(self, agg_weight):
+        self.agg_weight = agg_weight
 
     def isRegistered(self):
         return self.isRegistered
@@ -805,7 +819,7 @@ class BasicClient(Client):
 
         if client_state == ClientState.sg_ready:
             # Client received new SG weights before local training could complete
-            self.logger.debug(f'--- The training was too slow. A new SG model is available. ---')
+            self.logger.debug(f'--- A new SG model is available. ---')
             return False
         else:
             # Save trained model weights and performance metrics, then send to aggregator
@@ -817,7 +831,7 @@ class BasicClient(Client):
             self.tran_state(ClientState.sending)
             return True
 
-    def wait_for_sg_model(self):
+    def wait_for_sg_model(self, get_sg_model_id=False):
         # Semi-global model received by client
         while (self.read_agent_state() != ClientState.sg_ready):
             time.sleep(5)
@@ -833,7 +847,11 @@ class BasicClient(Client):
 
         # Change state to begin local training process
         self.tran_state(ClientState.training)
-        return merged_model
+        
+        if (get_sg_model_id):
+            return merged_model, sg_models['model_id']
+        else:
+            return merged_model
 
 class IntegratedClient(Client):
 
@@ -1005,7 +1023,6 @@ class IntegratedClient(Client):
 
             time.sleep(1)
 
-
 def run_client():
     """
         Required Configuration Parameters:
