@@ -152,7 +152,8 @@ class BaseModel(object):
 
     def create_serialized_obj(self):
         if self.type == BaseModelConvFormat.pytorch_format:
-            self.ser_obj = self.obj.state_dict()
+            state_dict = self.obj.state_dict()
+            self.ser_obj = {k:v.numpy() for k,v in state_dict.items()}
         elif self.type == BaseModelConvFormat.keras_format:
             self.ser_obj = None
         else:
