@@ -11,14 +11,14 @@ def generate_lmodel_update_message(agent_id: str,
                                    model_id: str,
                                    local_models: Dict[str, np.array],
                                    performance_dict: Dict[str, float],
-                                   num_samples: int) -> List[Any]:
+                                   agg_weight: float) -> List[Any]:
     update_rep = {
         "agent_id": agent_id,
         "model_id": model_id,
         "models": local_models,
         "gene_time": time.time(),
         "performance_dict": performance_dict,
-        "num_samples": num_samples
+        "agg_weight": agg_weight
     }
 
     msg = [AgentMsgType.update, update_rep]
@@ -66,4 +66,4 @@ def generate_initialize_model_message(agent, model_id, models, gene_time, perfor
 
     msg = [AgentMsgType.initialize_model, agent_rep]
 
-    return msg
+    return msg
\ No newline at end of file
